#pragma once

#include "GtPoint3.h"
#include "GtPoint4.h"

class GtMatrix4
{
public:
	union
	{
		struct
		{
			float	m11, m12, m13, m14;
			float	m21, m22, m23, m24;
			float	m31, m32, m33, m34;
			float	m41, m42, m43, m44;
		};

		float m[4][4];
		float loop[16];
	};

	GtMatrix4();
	GtMatrix4(const GtPoint3f &col0, const GtPoint3f& col1, const GtPoint3f& col2);
	GtMatrix4(const GtPoint4f &col0, const GtPoint4f& col1, const GtPoint4f& col2, const GtPoint4f& col3);
	GtMatrix4(float (*_m)[4]);
	GtMatrix4(const GtMatrix4& mat);
	GtMatrix4(float _11, float _12, float _13, float _14,
			  float _21, float _22, float _23, float _24,
			  float _31, float _32, float _33, float _34,
			  float _41, float _42, float _43, float _44);


	// Row  Լ
	inline void GetRow(unsigned int uiRow, GtPoint4f& row) const;
	inline void GetRow(unsigned int uiRow, GtPoint3f& row) const;
	inline void SetRow(unsigned int uiRow, const GtPoint4f& row);
	inline void SetRow(unsigned int uiRow, const GtPoint3f& row);
	inline void GetRow(unsigned int uiRow, float& f0, float& f1, float& f2) const;
	inline void SetRow(unsigned int uiRow, float f0, float f1, float f2);
	inline void GetRow(unsigned int uiRow, float* pRow) const;
	inline void SetRow(unsigned int uiRow, const float* pRow);

	// Column  Լ
	inline void GetCol(unsigned int uiCol, GtPoint4f& col) const;
	inline void GetCol(unsigned int uiCol, GtPoint3f& col) const;
	inline void SetCol(unsigned int uiCol, const GtPoint4f& col);
	inline void SetCol(unsigned int uiCol, const GtPoint3f& col);
	inline void GetCol(unsigned int uiCol, float& f0, float& f1, float& f2);
	inline void SetCol(unsigned int uiCol, float f0, float f1, float f2);
	inline void GetCol(unsigned int uiCol, float* pCol) const;
	inline void SetCol(unsigned int uiCol, const float* pCol);

	GtMatrix4		MakeTranspose(void);
	GtMatrix4		MakeInverse(void) const;
	void			MakeZero(void);
	void			MakeIdentity(void);
	void			MakeXRotation(float fAngle);
	void			MakeYRotation(float fAngle);
	void			MakeZRotation(float fAngle);
	void			MakeRotation(float fAngle, float x, float y, float z);
	void			MakeScaling(float x, float y, float z);
	void			MakeTranslation(float x, float y, float z);

	//  
	void			LookAtLH(GtPoint3f& ptEyePt,GtPoint3f& ptLookatPt,GtPoint3f& ptUpPt);
	void			LookAtRH(GtPoint3f& ptEyePt,GtPoint3f& ptLookatPt,GtPoint3f& ptUpPt);
	//   (ٹ)
	void			PerspectiveFovLH(const float fFovy,const float fAspect,const float fZnear,const float fZfar);
	void			PerspectiveFovRH(const float fFovy,const float fAspect,const float fZnear,const float fZfar);
	//   (翵)
	void			OrthoLH(const float fWidth,const float fHeight,const float fZnear,const float fZfar);
	void			OrthoRH(const float fWidth,const float fHeight,const float fZnear,const float fZfar);

	GtMatrix4		operator		+	(const GtMatrix4& mat) const;
	GtMatrix4		operator		-	(const GtMatrix4& mat) const;
	GtMatrix4		operator		*	(const float fValue) const;
	GtMatrix4		operator		*	(const GtMatrix4& mat) const;
	GtMatrix4		operator		/	(const GtMatrix4& mat) const;

	GtMatrix4		operator		+=	(const GtMatrix4& mat);
	GtMatrix4		operator		-=	(const GtMatrix4& mat);
	GtMatrix4		operator		*=	(const float fValue);
	GtMatrix4		operator		*=	(const GtMatrix4& mat);
	GtMatrix4		operator		/=	(const GtMatrix4& mat);

	GtMatrix4		operator		-	() const;

	bool			operator		==	(const GtMatrix4& mat) const;
	bool			operator		!=	(const GtMatrix4& mat) const;

	GtMatrix4&		operator		=	(const GtMatrix4& mat);
};

#include "GtMatrix4.inl"