#include "GtPlane.h"

GtPlane::GtPlane(const float _a,const float _b,const float _c,const float _d)
{
	a = _a; b = _b; c = _c; d = _d;
}
GtPlane::GtPlane(const GtPlane& _in)
{
	*this = _in;
}
GtPlane::GtPlane(const GtPoint3f& normal,const GtPoint3f pt)
{
	a = normal.x;
	b = normal.y;
	c = normal.z;
	d = -normal.Dot(pt);	
}

GtPlane::GtPlane(const GtPoint3f& _a, const GtPoint3f& _b, const GtPoint3f& _c)
{
	GtPoint3f pt = (_b - _a).Cross(_c - _a);
	GtPoint3f n = pt * (1.0f/pt.Length());
	a = n.x;
	b = n.y;
	c = n.z;
	d = -n.Dot(_a);	
}

float GtPlane::Dot(GtPoint3f& pt) const
{
	return (a * pt.x + b * pt.y + c * pt.z + d * 1.0f);
}

float GtPlane::DotCoord(GtPoint3f &pt) const
{
	return (a * pt.x + b * pt.y + c * pt.z) + d;
}

float GtPlane::DotNormal(GtPoint3f &pt) const
{
	return ( a*pt.x + b*pt.y + c*pt.z + d*0.0f );
}

void GtPlane::PlaneFromPointNormal(const GtPoint3f &normal, const GtPoint3f &pt)
{
	a = normal.x;
	b = normal.y;
	c = normal.z;
	d = -normal.Dot(pt);
}

void GtPlane::PlaneFromPoints(const GtPoint3f& _a,const GtPoint3f& _b,const GtPoint3f& _c)
{
	GtPoint3f pt = (_b - _a).Cross(_c - _a);
	GtPoint3f n = pt * (1.0f/pt.Length());
	a = n.x;
	b = n.y;
	c = n.z;
	d = -n.Dot(_a);
}

void GtPlane::PlaneIntersectLine(GtPoint3f *pOut, GtPoint3f &p1, GtPoint3f &p2)
{
	GtPoint3f pt1(a, b, c);
	GtPoint3f pt2 = p2 - p1;
	*pOut = p1 - pt2 * ((pt1.Dot(p1)+d) / (pt2.Dot(pt1)));
}

void GtPlane::Normalize(void)
{
	float q = 1 / GtSqrt(a * a + b * b + c * c);
	a = q * a;
	b = q * b;
	c = q * c;
	d = q * d;
}

GtPlane GtPlane::Normalized()
{
	GtPlane result;
	float q = 1 / GtSqrt(a * a + b * b + c * c);
	result.a = q * a;
	result.b = q * b;
	result.c = q * c;
	result.d = q * d;

	return result;
}

bool GtPlane::operator == (const GtPlane& in) const
{
	return (a == in.a && b == in.b && c == in.c && d == in.d );
}

bool GtPlane::operator != (const GtPlane& in) const
{
	return !(in == *this);
}

GtPlane& GtPlane::operator =( const GtPlane& in)
{
	this->a = in.a;
	this->b = in.b;
	this->c = in.c;
	this->d = in.d;

	return *this;
}

GtPlane GtPlane::operator -() const
{
	GtPlane result;
	result.a = -(this->a);
	result.b = -(this->b);
	result.c = -(this->c);
	result.d = -(this->d);

	return result;
}