#pragma once

#include "GtMath.h"

class GtMatrix4;

template<typename T>
class GtPoint3
{
public:
	T x, y, z;

	GtPoint3<T>() { }
	GtPoint3<T>(T x, T y, T z);

	inline const T& operator[] (int i) const;
	inline T& operator[] (int i);

	inline bool operator ==(const GtPoint3<T>& pt) const;
	inline bool operator !=(const GtPoint3<T>& pt) const;
	
	inline GtPoint3<T>	operator +(const GtPoint3<T>& pt) const;
	inline GtPoint3<T>	operator -(const GtPoint3<T>& pt) const;
	inline T			operator *(const GtPoint3<T>& pt) const;
	inline GtPoint3<T>	operator *(T tScalar) const;
	inline GtPoint3<T>	operator /(T tScalar) const;
	inline GtPoint3<T>	operator -() const;
	static GtPoint3<T>	ComponentProduct(const GtPoint3<T>& p0, const GtPoint3<T>& p1);

	inline GtPoint3<T>	operator +=(const GtPoint3<T>& pt);
	inline GtPoint3<T>	operator -=(const GtPoint3<T>& pt);
	inline GtPoint3<T>	operator *=(T tScalar);
	inline GtPoint3<T>	operator /=(T tScalar);

	// İ  
	inline void			TransformCoordColumn(const GtMatrix4& mat);		// Matrix * GtPoint3f
	inline void			TransformCoordRow(const GtMatrix4& mat);		// GtPoint3f * Matrix
	inline GtPoint3<T>	TransformNormalColums(const GtMatrix4& mat);
	inline GtPoint3<T>	TransformNormalRow(const GtMatrix4& mat);

	inline void			SetScale(const float value);
	inline void			SetScale(const float _x, const float _y, const float _z);
	inline void			SetRotateX(const float fAngle);
	inline void			SetRotateY(const float fAngle);
	inline void			SetRotateZ(const float fAngle);

	inline float		Length() const;
	inline float		SqrLength() const;
	inline float		Dot(const GtPoint3<T>& pt) const;
	inline float		Unitize();
	inline GtPoint3<T>	Normalize(void);
	
	inline GtPoint3<T>	Cross(const GtPoint3<T>& pt) const;
	inline GtPoint3<T>	UnitCross(const GtPoint3<T>& pt) const;

	inline GtPoint3<T>	Perpendicular() const;
};
typedef GtPoint3<int>		GtPoint3i;
typedef GtPoint3<float>		GtPoint3f;

#include "GtPoint3.inl"