#include "GtPoint3.h"
#include "GtMatrix4.h"

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T>::GtPoint3(T tX, T tY, T tZ)
{
	x = tX;
	y = tY;
	z = tZ;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline const T& GtPoint3<T>::operator [] (int i) const
{
	const T *base = &x;
	return (T&)base[i];
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline T& GtPoint3<T>::operator [] (int i)
{
	T *base = &x;
	return (T&)base[i];
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline bool GtPoint3<T>::operator ==(const GtPoint3<T>& pt) const
{
	return (x == pt.x && y == pt.y && z == pt.z);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline bool GtPoint3<T>::operator !=(const GtPoint3<T>& pt) const
{
	return !(*this == pt);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::operator +(const GtPoint3<T>& pt) const
{
	return GtPoint3<T>(x+pt.x, y+pt.y, z+pt.z);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::operator -(const GtPoint3<T>& pt) const
{
	return GtPoint3<T>(x-pt.x, y-pt.y, z-pt.z);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline T GtPoint3<T>::operator *(const GtPoint3<T>& pt) const
{
	return (x*pt.x + y*pt.y + z*pt.z);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::operator *(T tScalar) const
{
	return GtPoint3<T>(x*tScalar, y*tScalar, z*tScalar);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::operator /(T tScalar) const
{
	// / 꺸 *  Ƿ
	float fInvScalar = 1.0f/tScalar;
	return GtPoint3<T>(x*fInvScalar, y*fInvScalar, z*fInvScalar);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::operator -() const
{
	return GtPoint3<T>(-x, -y, -z);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::operator +=(const GtPoint3<T>& pt)
{
	x += pt.x;
	y += pt.y;
	z += pt.z;
	return *this;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::operator -=(const GtPoint3<T>& pt)
{
	x -= pt.x;
	y -= pt.y;
	z -= pt.z;
	return *this;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::operator *=(T tScalar)
{
	x *= tScalar;
	y *= tScalar;
	z *= tScalar;
	return *this;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::operator /=(T tScalar)
{
	float fInvScalar = 1.0f/fScalar;
	x *= fInvScalar;
	y *= fInvScalar;
	z *= fInvScalar;
	return *this;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::ComponentProduct(const GtPoint3<T> &p0, const GtPoint3<T> &p1)
{
	return GtPoint3<T>(p0.x * p1.x, p0.y * p1.y, p0.z * p1.z);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline float GtPoint3<T>::Length() const
{
	return GtSqrt((x*x) + (y*y) + (z*z));
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline float GtPoint3<T>::SqrLength() const
{
	return ((x*x) + (y*y) + (z*z));
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline float GtPoint3<T>::Dot(const GtPoint3<T> &pt) const
{
	return ((x*pt.x) + (y*pt.y) + (z*pt.z));
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline float GtPoint3<T>::Unitize()
{
    float fLength = Length();

    if ( fLength > 1e-06f )
    {
        float fRecip = 1.0f/fLength;
        x *= fRecip;
        y *= fRecip;
        z *= fRecip;
    }
    else
    {
        x = 0.0f;
        y = 0.0f;
        z = 0.0f;
        fLength = 0.0f;
    }

    return fLength;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::Normalize(void)
{
	GtPoint3<T> normal = *this;
	float fLength = Length();
	float fRecip = 1.0f/fLength;
	normal.x *= fRecip;
	normal.y *= fRecip;
	normal.z *= fRecip;
	return normal;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::Cross(const GtPoint3<T> &pt) const
{
    return GtPoint3<T>(y*pt.z-z*pt.y,z*pt.x-x*pt.z,x*pt.y-y*pt.x);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::UnitCross(const GtPoint3<T> &pt) const
{
    GtPoint3<T> cross(y*pt.z-z*pt.y,z*pt.x-x*pt.z,x*pt.y-y*pt.x);
    float fLength = cross.Length();
    if ( fLength > 1e-06f )
        return cross/fLength;
    else
        return GtPoint3<T>(0.0f,0.0f,0.0f);
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline GtPoint3<T> GtPoint3<T>::Perpendicular() const
{
	if(GtAbs(x) < GtAbs(y))
	{
		if(GtAbs(z) < GtAbs(x))
			return UnitCross(GtPoint3<T>(0, 0, 1));
		else
			return UnitCross(GtPoint3<T>(1, 0, 0));
	}
	else
	{
        if(GtAbs(z) < GtAbs(y))
            return UnitCross(GtPoint3<T>(0, 0, 1));
        else
            return UnitCross(GtPoint3<T>(0, 1, 0));
	}
}
//------------------------------------------------------------------------------------


//------------------------------------------------------------------------------------
template<typename T>
inline void GtPoint3<T>::TransformCoordColumn(const GtMatrix4& mat)
{
	T w = 1;
	GtPoint3<T> result;
	result.x = mat.m[0][0] * x + mat.m[0][1] * y + mat.m[0][2] * z + mat.m[0][3] * w;
	result.y = mat.m[1][0] * x + mat.m[1][1] * y + mat.m[1][2] * z + mat.m[1][3] * w;
	result.z = mat.m[2][0] * x + mat.m[2][1] * y + mat.m[2][2] * z + mat.m[2][3] * w;
	*this = result;
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
template<typename T>
inline void GtPoint3<T>::TransformCoordRow(const GtMatrix4& mat)
{
	T w = 1;
	GtPoint3<T> result;
	result.x = mat.m[0][0] * x + mat.m[1][0] * y + mat.m[2][0] * z + mat.m[3][0] * w;
	result.y = mat.m[0][1] * x + mat.m[1][1] * y + mat.m[2][1] * z + mat.m[3][1] * w;
	result.z = mat.m[0][2] * x + mat.m[1][2] * y + mat.m[2][2] * z + mat.m[3][2] * w;

	*this = result;
}
//------------------------------------------------------------------------------------