#pragma once

#include"GtPoint3.h"
#include"GtMatrix4.h"

class GtQuaternion
{
public:
	union
	{
		struct
		{
			float x, y, z, w;
		};
		float quat[4];
	};
	GtQuaternion(void) { };
	GtQuaternion(float _w, float _x, float _y, float _z);
	GtQuaternion(float fAngle, const GtPoint3f& axis);
	
	void	SetW(float _w);
	void	SetX(float _x);
	void	SetY(float _y);
	void	SetZ(float _z);
	float	GetW(void) const;
	float	GetX(void) const;
	float	GetY(void) const;
	float	GetZ(void) const;

	void SetValue(float _w, float _x, float _y, float _z);
	void GetValue(float& _w, float& _x, float& _y, float& _z);

	GtQuaternion operator +		(const GtQuaternion& q) const;
	GtQuaternion operator -		(const GtQuaternion& q) const;
	GtQuaternion operator -		(void) const;
	GtQuaternion operator *		(const GtQuaternion& q) const;
	GtQuaternion operator *		(float fValue) const;

	GtQuaternion operator +=	(const GtQuaternion& q);
	GtQuaternion operator -=	(const GtQuaternion& q);
	GtQuaternion operator *=	(const GtQuaternion& q);
	GtQuaternion operator *=	(const float& fValue);

	bool operator == (const GtQuaternion& q) const;
	bool operator != (const GtQuaternion& q) const;

	static	float					Dot(const GtQuaternion& p, const GtQuaternion& q);
	static	GtQuaternion			UnitInverse(const GtQuaternion& q);
	static	GtQuaternion			Exp(const GtQuaternion& q);
	static	GtQuaternion			Log(const GtQuaternion& q);

	//   
	static	GtQuaternion			Slerp(float t, const GtQuaternion& p, const GtQuaternion& q);
	static	GtQuaternion			Intermediate(const GtQuaternion& q0, const GtQuaternion& q1, const GtQuaternion& q2);
	static	GtQuaternion			Squad(float t, const GtQuaternion& p, const GtQuaternion& a, const GtQuaternion& b, const GtQuaternion& q);

	// Point  
	void	ToAngleAxis(float& angle, GtPoint3f& axis) const;
	void	FromAngleAxis(float angle, const GtPoint3f& axis);

	// ȸ
	void	FromAngleAxisX(float fAngle);
	void	FromAngleAxisY(float fAngle);
	void	FromAngleAxisZ(float fAngle);

	//	 İ 
	void	ToRotation(GtMatrix4& rot) const;
	void	FromRotation(const GtMatrix4& rot);

	void	Normalize(void);
	void	FastNormalize(void);

	// Slerp  Ǵ°
	static float ISqrt_approx_in_neighborhood(float s);
	static float Lerp(float v0, float v1, float fPerc);
	static float CounterWarp(float t, float fCos);
	

	static const GtQuaternion	IDENTITY;
	static const float			ms_fEpsilon;
};