#include "GtSoftwareRenderDevice.h"
#include<iostream>


GtSoftwareRenderDevice::GtSoftwareRenderDevice() : 
m_hWnd(NULL), 
m_hMainBuffer(NULL),
m_hBackBuffer(NULL),
m_hBitmap(NULL),
m_hDefaultBitmap(NULL),
m_pBit(NULL),
m_iWidth(0),
m_iHeight(0),
m_nPitch(0)
{
}

GtSoftwareRenderDevice::~GtSoftwareRenderDevice()
{
	SelectObject(m_hBackBuffer, m_hDefaultBitmap);
	DeleteDC(m_hBackBuffer);
	DeleteObject(m_hDefaultBitmap);
	DeleteObject(m_hBitmap);
	ReleaseDC(m_hWnd, m_hMainBuffer);
	SAFE_DELETE(m_pBit);
}

void GtSoftwareRenderDevice::Create(HWND hWnd, int iWidth, int iHeight)
{
	if(!hWnd)
		return ;

	m_hWnd				= hWnd;
	m_hMainBuffer		= GetDC(hWnd);
	m_hBackBuffer		= CreateCompatibleDC(m_hMainBuffer);
	m_hBitmap			= CreateBitmap(m_hMainBuffer, m_iWidth = iWidth, m_iHeight = iHeight, &m_pBit);
	
	m_hDefaultBitmap	= (HBITMAP)SelectObject(m_hBackBuffer, m_hBitmap);
}

void GtSoftwareRenderDevice::Swap()
{
	BitBlt(m_hMainBuffer, 0, 0, m_iWidth, m_iHeight, m_hBackBuffer, 0, 0, SRCCOPY);
}

void GtSoftwareRenderDevice::Update()
{
}

void GtSoftwareRenderDevice::SetPixel(int x, int y, int color)
{
	int i = y * m_nPitch + x * 3;
	
	m_pBit[i]		=		GT_GET_BLUE(color);
	m_pBit[i+1]		=		GT_GET_GREEN(color);
	m_pBit[i+2]		=		GT_GET_RED(color);
}

HBITMAP GtSoftwareRenderDevice::CreateBitmap(HDC hDC, int iWidth, int iHeight, BYTE **ppBit)
{
	BITMAPINFO BitInfo;

	ZeroMemory(&BitInfo, sizeof(BitInfo));
	BitInfo.bmiHeader.biSize		= sizeof (BITMAPINFOHEADER);
	BitInfo.bmiHeader.biWidth		= iWidth;
	BitInfo.bmiHeader.biHeight		= -iHeight;
	BitInfo.bmiHeader.biPlanes		= 1;
	BitInfo.bmiHeader.biBitCount	= 24;
	BitInfo.bmiHeader.biCompression	= BI_RGB;

	m_nPitch						= (m_iWidth * (BitInfo.bmiHeader.biBitCount >> 3) + 3) & ~3;

	return CreateDIBSection(hDC, (BITMAPINFO*)&BitInfo, DIB_RGB_COLORS, (VOID**)ppBit, NULL, 0);
}

void GtSoftwareRenderDevice::DrawLine(int x0, int y0, int x1, int y1, int color)
{
	if(!ClipLine(x0,y0,x1,y1,0,m_iWidth-1,0,m_iHeight-1)) return;

	int dx,dy,d,x,y,incX,incY;

	if(x0 > x1)
	{
		dx = x0 - x1;
		incX = -1;
	}
	else
	{
		dx = x1 - x0;
		incX = 1;
	}

	if(y0 > y1)
	{
		dy = y0 - y1;
		incY = -1;
	}
	else
	{
		dy = y1 - y0;
		incY = 1;
	}

	x = x0; y = y0;

	if(dx > dy) // -1 < m < 1   X 
	{
		dy <<= 1;
		d = dy - dx; // ʱ  dy - dx
		dx <<= 1;

		while (x != x1)
		{
			SetPixel(x,y,color);

			if (d >= 0) // NE κ 
			{
				//   (dy-dx) ε
				y += incY;
				d -= dx; // -dx  ϰ
			}
			d += dy; // dy ϰڴٴ 
			x += incX;
		}
	}
	else // |m| > 1   Y 
	{
		dx <<= 1;
		d = dy - dx;
		dy <<= 1;

		while(y != y1)
		{
			SetPixel(x,y,color);
			if (d < 0) // NE κ 
			{
				x += incX;
				d += dy;
			}
			d -= dx;

			y += incY;
		}
	}

	SetPixel(x,y,color);
}

bool GtSoftwareRenderDevice::ClipLine(int& x0, int& y0, int& x1, int &y1, int xmin, int xmax, int ymin, int ymax)
{
	int nCheck[2] = {0,0};
	int tx[2] = {x0,x1};
	int ty[2] = {y0,y1};

	for(int i = 0; i < 2; i++)
	{
		if     (ymin > ty[i]) nCheck[i] |= 8;
		else if(ymax < ty[i]) nCheck[i] |= 4;
		if     (xmin > tx[i]) nCheck[i] |= 1;
		else if(xmax < tx[i]) nCheck[i] |= 2;
	}

	if(!(nCheck[0] | nCheck[1])) return true; //  ȿ 
	if(nCheck[0] & nCheck[1])    return false;//  ȭ ۿ ־ ׸ʿ䰡 

	int sel = 0;

	while((nCheck[0] | nCheck[1]))
	{
		(nCheck[0] != 0) ? sel = 0 : sel = 1;

		if(nCheck[sel] & 8)
		{
			tx[sel] = (int)((((float)(tx[1] - tx[0]) / (ty[1] - ty[0])) * (ymin - ty[sel])) + tx[sel] + 0.5f);
			ty[sel] = ymin;

			nCheck[sel] -= 8;
		}
		else if(nCheck[sel] & 4)
		{
			tx[sel] = (int)((((float)(tx[1] - tx[0]) / (ty[1] - ty[0])) * (ymax - ty[sel])) + tx[sel] + 0.5f);
			ty[sel] = ymax;

			nCheck[sel] -= 4;
		}
		else if(nCheck[sel] & 2)
		{
			ty[sel] = (int)((((float)(ty[1] - ty[0]) / (tx[1] - tx[0])) * (xmax - tx[sel])) + ty[sel] + 0.5f);
			tx[sel] = xmax;

			nCheck[sel] -= 2;
		}
		else if(nCheck[sel] & 1)
		{
			ty[sel] = (int)((((float)(ty[1] - ty[0]) / (tx[1] - tx[0])) * (xmin - tx[sel])) + ty[sel] + 0.5f);
			tx[sel] = xmin;

			nCheck[sel] -= 1;
		}

		if     (ymin > ty[sel]) nCheck[sel] |= 8;
		else if(ymax < ty[sel]) nCheck[sel] |= 4;
		if     (xmin > tx[sel]) nCheck[sel] |= 1;
		else if(xmax < tx[sel]) nCheck[sel] |= 2;

		if(nCheck[0] & nCheck[1]) return false;//  ȭ ۿ ־ ׸ʿ䰡 
	}

	x0 = tx[0]; y0 = ty[0];
	x1 = tx[1]; y1 = ty[1];

	return true;
}

void GtSoftwareRenderDevice::Line_Horizon(int x1, int x2, int y, int color)
{
	if(y < 0 || y >= m_iHeight) return; // ׸ʿ䰡 .

	if(x1 > x2) std::swap(x1,x2);
	// Ŭ
	if(x1 < 0) x1 = 0;
	if(x2 >= m_iWidth) x2 = m_iWidth - 1;

	// x2 < 0 x1 >= m_lWidth  ˻ ʾƵ ȴ.
	//   Ʒ for ڵ false ó ޴´.

	int idx;
	for(int i = x1; i <= x2; i++)
	{
		//SetPixel(i,y,color);
		idx = y *  m_nPitch + i * 3;

		m_pBit[idx]     = GT_GET_BLUE(color);
		m_pBit[idx + 1] = GT_GET_GREEN(color);
		m_pBit[idx + 2] = GT_GET_RED(color);
	}	
}

void GtSoftwareRenderDevice::Line_Vertical(int x, int y1, int y2, int color)
{
	if(y1 > y2) std::swap(y1,y2);

	for(int i = y1; i <= y2; i++)
		SetPixel(x,y1++,color);
}

void GtSoftwareRenderDevice::Line_Horizon_Gouraud(int x1, int x2, int y, int color1, int color2)
{
	if(y < 0 || y >= m_iHeight) return; // ׸ʿ䰡 .

	if(x1 > x2)
	{
		std::swap(x1,x2);
		std::swap(color1,color2);
	}
	// Ŭ
	if(x1 < 0) x1 = 0;
	if(x2 >= m_iWidth) x2 = m_iWidth - 1;

	// x2 < 0 x1 >= m_iWidth  ˻ ʾƵ ȴ.
	//   Ʒ for ڵ false ó ޴´.

	int Length = x2 - x1;
	int mR;
	int mG;
	int mB;

	if(!Length)
	{
		mR = mG = mB = 0;
	}
	else
	{
		mR = (((int) GT_GET_RED(color2)   - GT_GET_RED(color1)) << 16) / Length;
		mG = (((int) GT_GET_GREEN(color2) - GT_GET_GREEN(color1)) << 16)  / Length;
		mB = (((int) GT_GET_BLUE(color2)  - GT_GET_BLUE(color1)) << 16) / Length;
	}

	int red   = GT_GET_RED(color1) << 16;
	int green = GT_GET_GREEN(color1) << 16;
	int blue  = GT_GET_BLUE(color1) << 16;

	static const int r = (1 << 15); // 0.5 .   

	int idx;
	for(int i = x1; i <= x2; i++)
	{
		//SetPixel(i,y,color);
		idx = y *  m_nPitch + i * 3;

		m_pBit[idx]     = (blue + r) >> 16;
		m_pBit[idx + 1] = (green + r) >> 16;
		m_pBit[idx + 2] = (red + r) >> 16;

		red   += mR;
		green += mG;
		blue  += mB;
	}
}

void GtSoftwareRenderDevice::GouraudShadingTopTriangle(int x1, int y1, int x2, int y2, int x3, int y3, int color1, int color2, int color3)
{
	if(y2 == y3 || y1 == y3)
		return;

	const int Height = y3 - y2;
	const int mLeft_Inv  = ((x3-x2) << 16) / Height;
	const int mRight_Inv = ((x3-x1) << 16) / Height;
	static const int r = (1 << 15); // 0.5 .   

	const int Length = y3 - y2;
	const int mLR = (((int) GT_GET_RED(color3)   - GT_GET_RED(color2)) << 16) / Length;
	const int mLG = (((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color2)) << 16)  / Length;
	const int mLB = (((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color2)) << 16) / Length;

	const int mRR = (((int) GT_GET_RED(color3)   - GT_GET_RED(color1)) << 16) / Length;
	const int mRG = (((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color1)) << 16)  / Length;
	const int mRB = (((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color1)) << 16) / Length;

	int LColor = color2;
	int LColorR = GT_GET_RED(color2) << 16;
	int LColorG = GT_GET_GREEN(color2) << 16;
	int LColorB = GT_GET_BLUE(color2) << 16;
	int RColor = color1;
	int RColorR = GT_GET_RED(color1) << 16;
	int RColorG = GT_GET_GREEN(color1) << 16;
	int RColorB = GT_GET_BLUE(color1) << 16;

	int xLeft = x2 << 16,xRight = x1 << 16;

	for(int i = y2; i <= y3; i++)
	{
		LColor = GT_RGB((LColorR + r) >> 16,(LColorG + r) >> 16,(LColorB + r) >> 16);
		RColor = GT_RGB((RColorR + r) >> 16,(RColorG + r) >> 16,(RColorB + r) >> 16);
		Line_Horizon_Gouraud((xLeft + r) >> 16,(xRight + r) >> 16,i,LColor,RColor);

		xLeft  += mLeft_Inv;
		xRight += mRight_Inv;

		LColorR += mLR;
		LColorG += mLG;
		LColorB += mLB;

		RColorR += mRR;
		RColorG += mRG;
		RColorB += mRB;
	}
}

void GtSoftwareRenderDevice::GouraudShadingBottomTriangle(int x1, int y1, int x2, int y2, int x3, int y3, int color1, int color2, int color3)
{
	if(y1 == y2 || y1 == y3)
		return;

	const int Height = y2 - y1;
	const int mLeft_Inv  = ((x2-x1) << 16) / Height;
	const int mRight_Inv = ((x3-x1) << 16) / Height;
	static const int r = (1 << 15); // 0.5 .   

	const int Length = y2 - y1;
	const int mLR = (((int) GT_GET_RED(color2)   - GT_GET_RED(color1)) << 16) / Length;
	const int mLG = (((int) GT_GET_GREEN(color2) - GT_GET_GREEN(color1)) << 16)  / Length;
	const int mLB = (((int) GT_GET_BLUE(color2)  - GT_GET_BLUE(color1)) << 16) / Length;

	const int mRR = (((int) GT_GET_RED(color3)   - GT_GET_RED(color1)) << 16) / Length;
	const int mRG = (((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color1)) << 16)  / Length;
	const int mRB = (((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color1)) << 16) / Length;

	int LColor = color1;
	int LColorR = GT_GET_RED(color1) << 16;
	int LColorG = GT_GET_GREEN(color1) << 16;
	int LColorB = GT_GET_BLUE(color1) << 16;
	int RColor = LColor;
	int RColorR = LColorR;
	int RColorG = LColorG;
	int RColorB = LColorB;


	int xLeft = x1 << 16,xRight = xLeft;

	for(int i = y1; i <= y2; i++)
	{
		LColor = GT_RGB((LColorR + r) >> 16,(LColorG + r) >> 16,(LColorB + r) >> 16);
		RColor = GT_RGB((RColorR + r) >> 16,(RColorG + r) >> 16,(RColorB + r) >> 16);

		Line_Horizon_Gouraud((xLeft + r) >> 16,(xRight + r) >> 16,i,LColor,RColor);

		xLeft  += mLeft_Inv;
		xRight += mRight_Inv;

		LColorR += mLR;
		LColorG += mLG;
		LColorB += mLB;

		RColorR += mRR;
		RColorG += mRG;
		RColorB += mRB;
	}
}

void GtSoftwareRenderDevice::GouraudShadingTriangle(int x1,int y1,int x2,int y2,int x3,int y3,int color1,int color2,int color3)
{
	if(y1 > y2)
	{
		std::swap(y1,y2);
		std::swap(x1,x2);
		std::swap(color1,color2);
	}
	if(y1 > y3)
	{
		std::swap(y1,y3);
		std::swap(x1,x3);
		std::swap(color1,color3);
	}
	if(y2 > y3)
	{
		std::swap(y2,y3);
		std::swap(x2,x3);
		std::swap(color2,color3);
	}

	if(y1 == y2) //   
	{
		GouraudShadingTopTriangle(x1,y1,x2,y2,x3,y3,color1,color2,color3);
	}
	else if(y2 == y3) // Ʒ  
	{
		GouraudShadingBottomTriangle(x1,y1,x2,y2,x3,y3,color1,color2,color3);
	}
	else // ,Ʒ  äϴ 
	{
		static const int r = (1 << 15); // 0.5 .   
		const int YLength = y2 - y1;
		const int NewX = (((x3-x1) << 16) / (y3-y1) * YLength + (x1 << 16) + r) >> 16;
		const int ColorLength = (y3 - y1);
		const int mNR = ((((int) GT_GET_RED(color3)   - GT_GET_RED(color1)) << 16)   / ColorLength) * YLength;
		const int mNG = ((((int) GT_GET_GREEN(color3) - GT_GET_GREEN(color1)) << 16) / ColorLength) * YLength;
		const int mNB = ((((int) GT_GET_BLUE(color3)  - GT_GET_BLUE(color1)) << 16)  / ColorLength) * YLength;
		const int NewColor = GT_RGB(((GT_GET_RED(color1)   << 16) + mNR + r) >> 16,
							        ((GT_GET_GREEN(color1) << 16) + mNG + r) >> 16,
									((GT_GET_BLUE(color1)  << 16) + mNB + r) >> 16);

		GouraudShadingBottomTriangle(x1,y1,x2,y2,NewX,y2,color1,color2,NewColor);
		GouraudShadingTopTriangle(NewX,y2,x2,y2,x3,y3,NewColor,color2,color3);
	}
}

void GtSoftwareRenderDevice::SetTransform(const GT_RENDERING_PIPELINE eState, const GtMatrix4 &mat)
{
	switch(eState)
	{
	case GT_WORLD:
		m_RenderPipeLine.matWorld = mat;
		break;
	case GT_VIEW:
		{
			GtMatrix4 RCa(mat);
			RCa.m41 = RCa.m42 = RCa.m43 = 0;
			RCa = mat / RCa;
			m_RenderPipeLine.ptCameraEye.x = -RCa.m41;
			m_RenderPipeLine.ptCameraEye.y = -RCa.m42;
			m_RenderPipeLine.ptCameraEye.z = -RCa.m43;
		}
		break;
	case GT_PROJECTION:
		m_RenderPipeLine.matProj = mat;
		break;
	}
}

bool GtSoftwareRenderDevice::SetViewPort(const sGT_ViewPort &pViewPort)
{
	m_RenderPipeLine.matViewPort.MakeIdentity();

	const float nCalcWidth  = (float)(pViewPort.Width  >> 1);
	const float nCalcHeight = (float)(pViewPort.Height >> 1);

	m_RenderPipeLine.matViewPort.m11 =  nCalcWidth;
	m_RenderPipeLine.matViewPort.m22 = -nCalcHeight;
	m_RenderPipeLine.matViewPort.m33 = pViewPort.MaxZ - pViewPort.MinZ;
	m_RenderPipeLine.matViewPort.m41 = pViewPort.x + nCalcWidth;
	m_RenderPipeLine.matViewPort.m42 = pViewPort.y + nCalcHeight;;
	m_RenderPipeLine.matViewPort.m43 = pViewPort.MinZ;

	return true;
}

bool GtSoftwareRenderDevice::GetViewPort(sGT_ViewPort &pOut)
{
	pOut.Width  = (unsigned long) (  m_RenderPipeLine.matViewPort.m11  * 2.f);
	pOut.Height = (unsigned long) ((-m_RenderPipeLine.matViewPort.m22) * 2.f);
	pOut.MinZ   = m_RenderPipeLine.matViewPort.m43;
	pOut.MaxZ   = m_RenderPipeLine.matViewPort.m33 + pOut.MinZ;
	pOut.x      = (unsigned long) (m_RenderPipeLine.matViewPort.m41 - m_RenderPipeLine.matViewPort.m11);
	pOut.y      = (unsigned long) (m_RenderPipeLine.matViewPort.m42 + m_RenderPipeLine.matViewPort.m22);

	return true;
}

bool GtSoftwareRenderDevice::CreateMatrixClipping(const sGT_Clipping &pClip)
{
	m_RenderPipeLine.matClip.MakeIdentity();

	m_RenderPipeLine.matClip.m11 = 2.f / pClip.ClipWidth;
	m_RenderPipeLine.matClip.m22 = 2.f / pClip.ClipHeight;
	m_RenderPipeLine.matClip.m33 = 1.f / (pClip.MaxZ - pClip.MinZ);
	m_RenderPipeLine.matClip.m41 = -1 - 2 * pClip.ClipX / pClip.ClipWidth;
	m_RenderPipeLine.matClip.m42 =  1 - 2 * pClip.ClipY / pClip.ClipHeight;
	m_RenderPipeLine.matClip.m43 = -pClip.MinZ / (pClip.MaxZ - pClip.MinZ);

	return true;
}

void GtSoftwareRenderDevice::DrawPrimitive(GT_PRIMITIVETYPE ePrimitiveState, unsigned int iStartVertex, unsigned int iPrimitiveCount)
{
	RenderingPipeLine();
}

bool GtSoftwareRenderDevice::BackFaceCullingLH(const GtPoint3f &p1, const GtPoint3f &p2, const GtPoint3f &p3)
{
	static GtPoint3f n,l;
	n = (p3 - p1).Cross(p2 - p1); // v cross u

	l.x = m_RenderPipeLine.ptCameraEye.x - p1.x;
	l.y = m_RenderPipeLine.ptCameraEye.y - p1.y;
	l.z = m_RenderPipeLine.ptCameraEye.z - p1.z;

	if(n.Dot(l) < 0.f)
		return false;

	return true;
}

bool GtSoftwareRenderDevice::BackFaceCullingRH(const GtPoint3f &p1, const GtPoint3f &p2, const GtPoint3f &p3)
{
	static GtPoint3f n,l;
	n = (p2 - p1).Cross(p3 - p1); // u cross v

	l.x = m_RenderPipeLine.ptCameraEye.x - p1.x;
	l.y = m_RenderPipeLine.ptCameraEye.y - p1.y;
	l.z = m_RenderPipeLine.ptCameraEye.z - p1.z;

	if(l.Dot(n) < 0.f)
		return false;

	return true;
}

bool GtSoftwareRenderDevice::RenderingPipeLine()
{
/*	if(!(m_FVF.IsFVF(GT_FVF_XYZRHW)))
	{
		Local_To_World_Object();
		if(!World_To_Camera_Object())
			return false;
		Camera_To_Proj_Object();
	}
	if(m_RenderStateFlag.bClip)
		Proj_Object_To_Clipping(); //  Ʈȯ(ȭ )

	Clipping_To_ViewPort();
	Fill_Rasterize();
*/
	return true;
}

bool GtSoftwareRenderDevice::Local_To_World_Object(void)
{
	//m_RenderObj.VtxBuff->m_vWorld.TransformCoordRow(m_RendPipe.matWorld);
/*	m_RenderPipeLine.matTotal = m_RenderPipeLine.matWorld;

	// ĸ 
	void* t = m_RenderObj.VertexBuffer->m_pVB;

	if(m_FVF.IsFVF(GT_FVF_XYZ))
	{
	//	static char* pVoid;
		static GtPoint3f t_xyz[3];
	//	pVoid = (char*)t;
		m_FVF.ResetPtr();

		m_RenderObj.bVisible.resize(m_RenderObj.VertexCount/3);
		for(unsigned int i = 0; i < m_RenderObj.VertexCount/3; i++)
		{
			for(unsigned int j = 0; j < 3; j++)
			{
				t_xyz[j] = *(GtPoint3f*) m_FVF.NextValue(GT_FVF_NONE,GT_FVF_XYZ);
				m_FVF.NextNode();
			}

			/// test
			t_xyz[0].TransformCoordRow(m_RenderPipeLine.matTotal);
			t_xyz[1].TransformCoordRow(m_RenderPipeLine.matTotal);
			t_xyz[2].TransformCoordRow(m_RenderPipeLine.matTotal);
			///

			m_RenderObj.bVisible[i] = BackFaceCullingRH(t_xyz[0],t_xyz[1],t_xyz[2]);
		}
	}
*/
	return true;
}


bool GtSoftwareRenderDevice::World_To_Camera_Object(void)
{
	return true;
}