#pragma once

#include "GtFVF.h"
#include "GtSoftwareRender_Type.h"
#include <windows.h>

class GtSoftwareRenderDevice
{
private:
	HWND			m_hWnd;
	HDC				m_hMainBuffer;
	HDC				m_hBackBuffer;
	HBITMAP			m_hBitmap;
	HBITMAP			m_hDefaultBitmap;
	BYTE*			m_pBit;
	int				m_iWidth;
	int				m_iHeight;
	int				m_nPitch;

private:
	GT_RenderPipeLine	m_RenderPipeLine;
	GtFVF				m_FVF;
	sGtRenderStateFlag	m_RenderStateFlag;
	sRenderObj			m_RenderObj;
	
public:
	GtSoftwareRenderDevice(void);
	~GtSoftwareRenderDevice(void);

public:
	void			Create(HWND hWnd, int iWidth, int iHeight);
	void			Destroy(void);
	void			Swap(void);

	void			Update(void);
	HBITMAP			CreateBitmap(HDC hDC, int iWidth, int iHeight, BYTE** ppBit);
	void			DrawFPS(void);

public:
	void			SetPixel(int x, int y, int color);
	void			DrawLine(int x0, int y0, int x1, int y1, int color);	// Breseham 
	bool			ClipLine(int& x0, int& y0, int& x1, int &y1, int xmin, int xmax, int ymin, int ymax);	//	Cohen Sutherland 
	void			Line_Horizon(int x1, int x2, int y, int color);		// ܻ
	void			Line_Vertical(int x, int y1, int y2, int color);	// ܻ

public:
	void			Line_Horizon_Gouraud(int x1,int x2,int y,int color1,int color2); //
	void			GouraudShadingTopTriangle(int x1,int y1,int x2,int y2,int x3,int y3,int color1,int color2,int color3);
	void			GouraudShadingBottomTriangle(int x1,int y1,int x2,int y2,int x3,int y3,int color1,int color2,int color3);
	void			GouraudShadingTriangle(int x1,int y1,int x2,int y2,int x3,int y3,int color1,int color2,int color3);
public:
	//  ̵ Z  
	void			Line_Horizon_GouraudZ(int x1,int x2,int y,unsigned int z1,unsigned int z2,int color1,int color2,unsigned int u1,unsigned int v1,unsigned int u2,unsigned int v2,unsigned int invZ1,unsigned int invZ2);
	void			GouraudShadingTopTriangleZ(int x1,int y1,unsigned int z1,int x2,int y2,unsigned int z2,int x3,int y3,unsigned int z3,int color1,int color2,int color3,unsigned int u1,unsigned int v1,unsigned int u2,unsigned int v2,unsigned int u3,unsigned int v3,unsigned int invZ1,unsigned int invZ2,unsigned int invZ3);
	void			GouraudShadingBottomTriangleZ(int x1,int y1,unsigned int z1,int x2,int y2,unsigned int z2,int x3,int y3,unsigned int z3,int color1,int color2,int color3,unsigned int u1,unsigned int v1,unsigned int u2,unsigned int v2,unsigned int u3,unsigned int v3,unsigned int invZ1,unsigned int invZ2,unsigned int invZ3);
	void			GouraudShadingTriangleZ(int x1,int y1,unsigned int z1,int x2,int y2,unsigned int z2,int x3,int y3,unsigned int z3,int color1,int color2,int color3,unsigned int u1,unsigned int v1,unsigned int u2,unsigned int v2,unsigned int u3,unsigned int v3,unsigned int invZ1,unsigned int invZ2,unsigned int invZ3);
public:
	// , ,   ޾ƿ 
	void			SetTransform(const GT_RENDERING_PIPELINE eState, const GtMatrix4& mat);
	void			DrawPrimitive(GT_PRIMITIVETYPE ePrimitiveState, unsigned int iStartVertex, unsigned int iPrimitiveCount);
	void			DrawPrimitiveUp(GT_PRIMITIVETYPE ePrimitiveState, unsigned int iPrimivieCount, const void* pVtxStreamZeroData, unsigned int iVtxStreamZeroStride);
	// Ʈ   ´.   Ʈ    ü̴.
	bool			SetViewPort(const sGT_ViewPort& pViewPort);
	bool			GetViewPort(sGT_ViewPort& pOut);

	// Ŭ  .
	bool			CreateMatrixClipping(const sGT_Clipping& pClip);

	// ĸ
	bool			BackFaceCullingLH(const GtPoint3f& p1, const GtPoint3f& p2, const GtPoint3f& p3);
	bool			BackFaceCullingRH(const GtPoint3f& p1, const GtPoint3f& p2, const GtPoint3f& p3);

	bool			RenderingPipeLine(void);
	bool			Local_To_World_Object(void);		//  
	bool			World_To_Camera_Object(void);		// 忡 ī޶
	bool			Camera_To_Proj_Object(void);		// ī޶󿡼 
	bool			Proj_Object_To_Clipping(void);		//  Ŭ
	bool			Clipping_To_ViewPort(void);			//  Ʈ ȯ(ȭ )
	bool			Fill_Rasterize(void);				//  ȭ  ﰢ ä.
};