#pragma once

#include"GtMatrix4.h"
#include"GtVertexBuffer.h"
#include<vector>

#define	SAFE_DELETE_ARRAY(p) { if(p) { delete[] (p);   (p)=NULL; } }	//  Ҵ 迭 	
#define	SAFE_DELETE(p) { if(p) { delete (p);   (p)=NULL; } }			//  Ҵ 	

#define GT_RGB(r,g,b) ((unsigned char)(r)|((unsigned short)(unsigned char)(g)<<8)|((unsigned long)(unsigned char)(b)<<16))

#define GT_GET_RED(color)		((unsigned char) (color))
#define GT_GET_GREEN(color)		((unsigned char) ((color) >> 8))
#define GT_GET_BLUE(color)		((unsigned char) ((color) >> 16))

typedef enum eGtRenderingPipeLine
{
	GT_WORLD		=	1,
	GT_VIEW			=	2,
	GT_PROJECTION	=	4
} GT_RENDERING_PIPELINE;

typedef enum eGtPrimitivetype
{
	GT_POINTLIST		=	1,
	GT_LINELIST			=	2,
	GT_LINESTRIP		=	3,
	GT_TRIANGLELIST		=	4,
	GT_TRIANGLESTROP	=	5,
	GT_TRIANGLEFAN		=	6,
	GT_FORCE_DWORD		=	0x7fffffff

} GT_PRIMITIVETYPE;

typedef struct sGtRenderingPipeLine
{
	GtMatrix4		matWorld;
	GtMatrix4		matView;
	GtMatrix4		matProj;
	GtMatrix4		matViewPort;
	GtMatrix4		matClip;
	GtPoint3f		ptCameraEye;
	GtMatrix4		matTotal;

	sGtRenderingPipeLine(void)
	{
		matWorld.MakeIdentity();
		matView.MakeIdentity();
		matProj.MakeIdentity();
		matViewPort.MakeIdentity();
		matClip.MakeIdentity();
		matTotal.MakeIdentity();
	}
}sGT_Rendering_PipeLine, GT_RenderPipeLine, *LPGT_Rendering_PipeLine;


typedef struct sGtRenderStateFlag
{
	unsigned int bSpecularEnable : 1;
	unsigned int bLighting : 1;
	unsigned int bNormalizeNormals : 1;
	unsigned int bClip : 1;
	unsigned int bZEnable : 1;
	unsigned int ShadeMode : 2;
	unsigned int DiffuseMaterialSource : 2;
	unsigned int SpecularMaterialSource : 2;
	unsigned int AmbientMaterialSource : 2;
	unsigned int EmissiveMaterialSource : 2;
	unsigned long lAmbient;

	sGtRenderStateFlag()
	{
		bSpecularEnable		   = 0;
		bLighting			   = 1;
		bNormalizeNormals	   = 0;
		lAmbient			   = 0;
		bClip				   = 0;
		ShadeMode			   = 1;
		bZEnable			   = 1;
		DiffuseMaterialSource  = 0;
		SpecularMaterialSource = 0;
		AmbientMaterialSource  = 0;
		EmissiveMaterialSource = 0;
	}
} sGtRenderStateFlag;


typedef struct sGtViewPort
{
	unsigned long x;		// » ȭ x ǥ 
	unsigned long y;		// » ȭ y ǥ 
	unsigned long Width;	// ϴ ȭ x ǥ
	unsigned long Height;	// ϴ ȭ y ǥ
	float		  MinZ;		//   ּ ( 0.0f)
	float		  MaxZ;		//   ִ ( 1.0f)
} sGT_ViewPort, *LPGT_ViewPort;

typedef struct sGtCliping
{
	float ClipX;			// ߸  ǥ » x 
	float ClipWidth;		// ߸  ȭ ʺ
	float ClipY;			// ߸  ǥ » y 
	float ClipHeight;		// ߸  ȭ 
	float MinZ;				//   ּ 
	float MaxZ;				//   ִ 
} sGT_Clipping;

typedef struct sGT_ColorValue
{
	float r;
	float g;
	float b;
	float a;

	sGT_ColorValue(const float _r = 0, const float _g = 0, const float _b = 0, const float _a = 1.0f) : r(_r), g(_g), b(_b), a(_a) {}
	sGT_ColorValue sGT_ColorValue::operator * (const float in) const
	{
		return sGT_ColorValue(r * in, g * in, b * in, a * in);
	}
	sGT_ColorValue sGT_ColorValue::operator * (const sGT_ColorValue& in) const
	{
		return sGT_ColorValue(r * in.r, g * in.g, b * in.b, a * in.a);
	}
	sGT_ColorValue sGT_ColorValue::operator + (const sGT_ColorValue& in) const
	{
		return sGT_ColorValue(r + in.r, g + in.g, b + in.b, a + in.a);
	}
	sGT_ColorValue sGT_ColorValue::operator += ( const sGT_ColorValue& in)
	{
		r += in.r;
		g += in.g;
		b += in.b;
		a += in.a;

		if( r > 1.0f ) r = 1.0f;
		if( g > 1.0f ) g = 1.0f;
		if( b > 1.0f ) b = 1.0f;
		if( a > 1.0f ) a = 1.0f;

		return (*this);
	}
} GT_COLORVALUE;

typedef enum eGT_RENDERSTATE
{
	GT_RS_ZENABLE                   = 7,    
	GT_RS_FILLMODE                  = 8,    
	GT_RS_SHADEMODE                 = 9,    
	GT_RS_ZWRITEENABLE              = 14,   
	GT_RS_ALPHATESTENABLE           = 15,   
	GT_RS_LASTPIXEL                 = 16,   
	GT_RS_SRCBLEND                  = 19,   
	GT_RS_DESTBLEND                 = 20,   
	GT_RS_CULLMODE                  = 22,   
	GT_RS_ZFUNC                     = 23,   
	GT_RS_ALPHAREF                  = 24,   
	GT_RS_ALPHAFUNC                 = 25,   
	GT_RS_DITHERENABLE              = 26,   
	GT_RS_ALPHABLENDENABLE          = 27,   
	GT_RS_FOGENABLE                 = 28,   
	GT_RS_SPECULARENABLE            = 29,   
	GT_RS_FOGCOLOR                  = 34,   
	GT_RS_FOGTABLEMODE              = 35,   
	GT_RS_FOGSTART                  = 36,   
	GT_RS_FOGEND                    = 37,   
	GT_RS_FOGDENSITY                = 38,   
	GT_RS_RANGEFOGENABLE            = 48,   
	GT_RS_STENCILENABLE             = 52,   
	GT_RS_STENCILFAIL               = 53,   
	GT_RS_STENCILZFAIL              = 54,   
	GT_RS_STENCILPASS               = 55,   
	GT_RS_STENCILFUNC               = 56,   
	GT_RS_STENCILREF                = 57,   
	GT_RS_STENCILMASK               = 58,   
	GT_RS_STENCILWRITEMASK          = 59,   
	GT_RS_TEXTUREFACTOR             = 60,   
	GT_RS_WRAP0                     = 128,  
	GT_RS_WRAP1                     = 129,  
	GT_RS_WRAP2                     = 130,  
	GT_RS_WRAP3                     = 131,  
	GT_RS_WRAP4                     = 132,  
	GT_RS_WRAP5                     = 133,  
	GT_RS_WRAP6                     = 134,  
	GT_RS_WRAP7                     = 135,  
	GT_RS_CLIPPING                  = 136,
	GT_RS_LIGHTING                  = 137,
	GT_RS_AMBIENT                   = 139,
	GT_RS_FOGVERTEXMODE             = 140,
	GT_RS_COLORVERTEX               = 141,
	GT_RS_LOCALVIEWER               = 142,
	GT_RS_NORMALIZENORMALS          = 143,
	GT_RS_DIFFUSEMATERIALSOURCE     = 145,
	GT_RS_SPECULARMATERIALSOURCE    = 146,
	GT_RS_AMBIENTMATERIALSOURCE     = 147,
	GT_RS_EMISSIVEMATERIALSOURCE    = 148,
	GT_RS_VERTEXBLEND               = 151,
	GT_RS_CLIPPLANEENABLE           = 152,
	GT_RS_POINTSIZE                 = 154,
	GT_RS_POINTSIZE_MIN             = 155,
	GT_RS_POINTSPRITEENABLE         = 156,
	GT_RS_POINTSCALEENABLE          = 157,
	GT_RS_POINTSCALE_A              = 158,
	GT_RS_POINTSCALE_B              = 159,
	GT_RS_POINTSCALE_C              = 160,
	GT_RS_MULTISAMPLEANTIALIAS      = 161,
	GT_RS_MULTISAMPLEMASK           = 162,
	GT_RS_PATCHEDGESTYLE            = 163,
	GT_RS_DEBUGMONITORTOKEN         = 165,
	GT_RS_POINTSIZE_MAX             = 166,
	GT_RS_INDEXEDVERTEXBLENDENABLE  = 167,
	GT_RS_COLORWRITEENABLE          = 168,  
	GT_RS_TWEENFACTOR               = 170,  
	GT_RS_BLENDOP                   = 171,  
	GT_RS_POSITIONDEGREE            = 172,  
	GT_RS_NORMALDEGREE              = 173,  
	GT_RS_SCISSORTESTENABLE         = 174,
	GT_RS_SLOPESCALEDEPTHBIAS       = 175,
	GT_RS_ANTIALIASEDLINEENABLE     = 176,
	GT_RS_MINTESSELLATIONLEVEL      = 178,
	GT_RS_MAXTESSELLATIONLEVEL      = 179,
	GT_RS_ADAPTIVETESS_X            = 180,
	GT_RS_ADAPTIVETESS_Y            = 181,
	GT_RS_ADAPTIVETESS_Z            = 182,
	GT_RS_ADAPTIVETESS_W            = 183,
	GT_RS_ENABLEADAPTIVETESSELLATION = 184,
	GT_RS_TWOSIDEDSTENCILMODE       = 185,  
	GT_RS_CCW_STENCILFAIL           = 186,  
	GT_RS_CCW_STENCILZFAIL          = 187,  
	GT_RS_CCW_STENCILPASS           = 188,  
	GT_RS_CCW_STENCILFUNC           = 189,  
	GT_RS_COLORWRITEENABLE1         = 190,  
	GT_RS_COLORWRITEENABLE2         = 191,  
	GT_RS_COLORWRITEENABLE3         = 192,  
	GT_RS_BLENDFACTOR               = 193,  
	GT_RS_SRGBWRITEENABLE           = 194,  
	GT_RS_DEPTHBIAS                 = 195,
	GT_RS_WRAP8                     = 198,  
	GT_RS_WRAP9                     = 199,
	GT_RS_WRAP10                    = 200,
	GT_RS_WRAP11                    = 201,
	GT_RS_WRAP12                    = 202,
	GT_RS_WRAP13                    = 203,
	GT_RS_WRAP14                    = 204,
	GT_RS_WRAP15                    = 205,
	GT_RS_SEPARATEALPHABLENDENABLE  = 206,  
	GT_RS_SRCBLENDALPHA             = 207,  
	GT_RS_DESTBLENDALPHA            = 208,  
	GT_RS_BLENDOPALPHA              = 209,  

	GT_RS_FORCE_DWORD               = 0x7fffffff, /* force 32-bit size enum */
}GT_RENDERSTATE;

typedef enum eGT_SHADEMODE
{
	GT_SM_FLAT			=		1,
	GT_SM_GOURAUD		=		2,
	GT_SM_PHONG			=		3,
	GT_SM_FORCE_DWORD	=		0x7fffffff
}GT_SHADEMODE;

typedef struct sRenderObjectInfo
{
	LPGTVertexBuffer	VertexBuffer;
	std::vector<bool>	bVisible;
	unsigned int		VertexCount;
}sRenderObj;